import os
import subprocess
from tempfile import mkstemp
from lxml import etree

from pkg_resources import resource_filename


CLASSPATH = ':'.join([
    resource_filename('codaxmli', 'java/router.jar'),
    resource_filename('codaxmli', 'java/xercesImpl.jar'),
])


class Client(object):

    def __init__(self, host, port, lsv_name, user, password, company,
                 trace=False):
        self.host = host
        self.port = port
        self.lsv_name = lsv_name
        self.user = user
        self.password = password
        self.company = company
        self.trace = trace

    def send(self, request):
        if not isinstance(request, basestring):
            request = etree.tostring(request)
        if self.trace:
            print request
        request_fd, request_path = mkstemp(prefix='xmli-')
        os.write(request_fd, request)
        os.close(request_fd)

        response_fd, response_path = mkstemp(prefix='xmli-')
        subprocess.check_call([
            'java', '-classpath', CLASSPATH, 'com.coda.xml.router.Router',
            '-nsv:' + self.host, '-port:' + str(self.port),
            '-name:' + self.lsv_name, '-user:' + self.user,
            '-password:' + self.password, '-company:' + self.company,
            request_path
        ], stdout=response_fd)
        os.close(response_fd)

        response = etree.parse(response_path)
        os.remove(request_path)
        os.remove(response_path)

        errors = response.findall('//Reason')
        if errors:
            msgs = []
            for error in errors:
                lines = []
                for text in error.findall('Text'):
                    lines.append(text.text)
                for path in error.findall('Path'):
                    lines.append('Path: ' + path.text)
                for hint in error.findall('Hint'):
                    lines.append(hint.text)
                msgs.append('\n'.join(lines))
            raise XMLiError('\n\n'.join(msgs).encode('utf-8'))

        return response


class XMLiError(Exception):
    pass
